#!/usr/bin/python3
#-*- coding: utf-8 -*-



import operator, collections, os, sys, re, string, nltk
from nltk.tokenize import word_tokenize
from nltk.corpus import stopwords
from nltk.util import pr
from tabulate import tabulate

import pandas as pd
import numpy as np

rootdir = os.path.expanduser(os.environ['vault_path'])
target = os.environ['file']
# sys.stdout.write(target)

## README
# This script was originally written by macedotavares https://github.com/macedotavares/python-text-similarity
# I have adapted it slightly for my needs within my workflow

# My first attempt at creating a script that suggests similar notes.
# Place it at the root of your notes folder. `cd` to it and run the script with:
# `python3 similarity.py path/to/your/target/file.md`



similarity_threshold = 0.1

stop_words = set(stopwords.words('english')).union(set(stopwords.words('english')))
md_regex = re.compile(r'.*md$')
tag_regex = re.compile(r'#([-_\d\w]+)')
link_regex = re.compile(r'\[\[(.*?)[|#]')


def read(file):
    with open(file, 'r') as f:
        content = f.read()
        f.close()
    return content.lower()

def remove_numbers(input_str):
    result = re.sub(r'\d+', '', input_str)
    return result

def remove_whitespace(input_str):
    result = re.sub(r'[\t\n\r]', '', input_str)
    return result    

def remove_punctuation(input_str):
    result = input_str.translate(str.maketrans('', '', string.punctuation))
    return result

def remove_stopwords(input_str):
    tokens = word_tokenize(input_str)
    result = [i for i in tokens if not i in stop_words]
    return result

def preprocess(file):
    return remove_stopwords(remove_whitespace(remove_punctuation(remove_numbers(read(file)))))

def extract_keywords(input_str):
	tags = set(re.findall(tag_regex, input_str))
	links = set(re.findall(link_regex, input_str))
	return tags.union(links)

def jaccard_similarity(list1, list2): 
	intersection = set(list1).intersection(set(list2))
	union = set(list1).union(set(list2))
	if len(union)>0:
		return float(len(intersection) / len(union))
	else: return 0.0

def df_to_markdown(df, y_index=False):
    blob = tabulate(df, headers='keys', tablefmt='pipe')
    if not y_index:
        # Remove the index with some creative splicing and iteration
        return '\n'.join(['| {}'.format(row.split('|', 2)[-1]) for row in blob.split('\n')])
    return blob

raw_target = read(target)
# print(str(sys.argv))

preprocessed_target = preprocess(target)
table = []

for root, dirs, files in os.walk(rootdir):
	for other_file in files:
		if md_regex.match(other_file):
			other_file_path = os.path.join(root, other_file)
			preprocessed_other = preprocess(other_file_path)
			raw_other = read(other_file_path)
			filename = other_file

			filename = filename[:-3]
			# filename = "[["+filename+"]]"

			text_similarity = jaccard_similarity(preprocessed_target, preprocessed_other)
			

			keyword_similarity = jaccard_similarity(extract_keywords(raw_target), extract_keywords(raw_other))
			
			# print(extract_keywords(raw_other))
			similarity = (0.25*text_similarity) + (0.75*keyword_similarity)
			#print(similarity)
			if similarity > similarity_threshold:
				# print(other_file)
				# print(text_similarity)
				# print(preprocessed_other)

				row = [filename, round(similarity,3)]
				table.append(row)

data = np.array(table)
df = pd.DataFrame(data, columns=['filename', 'score'])
df = df.sort_values(by=['score'], ascending = False)
df = df.drop_duplicates()
# print(df)

tableOutput = "## Similar Notes \n"
tableOutput += df_to_markdown(df)
tableOutput += "\n"
# print(tableOutput)
sys.stdout.flush()

sys.stdout.write(tableOutput)
# print(tabulate(table, headers=['File', 'Score']))
